/* objects.h
 * Copyright (C) 2004 Djukic Marko, Pernus Klemen, Zagar Miha
 * For conditions of distribution and use, see the copyright notice in main.cpp
 */
#ifndef _OBJECT_H_
#define _OBJECT_H_

#include "fx.h"
#include "primitive.h"
#include "constraints.h"
#include "color.h"

#include <math.h>
#include <list>
using namespace std;


// *************************************
//
//	Razred OBJECT ... oce vseh objektov
//
// *************************************
class Object: public Primitive {
protected:
	//flag pri preverjanju skupine crt (pri recheck_constraints)
	int recheck;
	
	//flag pri premikanju
	int already_moved;
	
	//barva objekta
	Color *object_color;
	//debelina objekta
	unsigned char object_width;
	unsigned char closest_vertex;
	int changed;
	
	//tekst ki se izpise ob narianju objekta
	char *text;
public:
	//flag pri spremembi objekta
	list <Constraint*> constraints;

	Object();
	~Object();
	
	//uporablja se pri risanju
	//nastavi debelino objekta
	void set_object_width(unsigned char w);
	//vrne debelino objekta
	unsigned char get_object_width();
	//nastavi barvo objekat
	void set_object_color(Color *c);
	//vrne barvo objekta
	Color *get_object_color();
	virtual unsigned int get_vertex_distance(Point *tocka) {return 0;};
	unsigned char get_closest_vertex();
	
	//********************************//
	//vrne 1 ali 0 ce tocka je na tocki ali ne ... postavi
	//closest_vertex, tako da lahoko pogledas z get_closest_vertex
	virtual unsigned char on_vertex(Point *tocka) {return 0;};
	//premikanje objekta
	virtual int move_object_by(Point *dp) {return 1;};
	//prmaken tocka obekta za razliko tocke
	virtual int move_vertex_by(unsigned char vertex, Point *dp) {return 1;};
	//premakne tocka objekta NA tocko
	virtual int move_vertex_to(unsigned char vertex, Point *p) {return 1;};
	//vrne koordinate tocke vertex
	virtual Point *get_vertex_point(unsigned char vertex) {return NULL;}; 

	virtual Point *get_other_vertex_point (int vertex) {return NULL;};

	//pobrise visited_on_move flag pri omejitvah (postavi se pri preverjanju - da ne pride do nesk. zanke)
	void clear_visited_flag();
	//vrne recheck flag
	int get_recheck() {return recheck;};
	//dolocanje vrednosti recheck flaga
	void set_recheck(int r){recheck = r;};
	
	int get_changed() {return changed;};
	void set_changed(int c) {changed = c;};

	//vrne already_moved_flag
	int get_already_moved() {return already_moved;};
	//doloca vrednost already_moved flaga
	void set_already_moved(int a) {already_moved = a;};

	//izbris omejitve: SAMO IZ CONSTRAINT MANAGERJA!!!
	void break_constraint(Constraint *c);
	//dodajanje omejitve: SAMO IZ CM
	void add_constraint(Constraint *c);

	//vrne in nastavi text
	char *get_text();
	void set_text(char *);
};

// *************************************
//
//	Razred LINE
//
// *************************************
class Line: public Object {
//private:
public:
	//zacetna in koncna tocka crte.
	Point *T1, *T2;
	//koeficienti za implicitno obliko
	int a,b,c;

public:
	//ustvari crto med tockama t1 in t2;
	Line(Point *t1, Point *t2);
	~Line();
	//virtual iz object
	unsigned char on_vertex(Point *tocka);
	//virtual iz object
	int move_object_by(Point *tocka);
	//virtual iz object
	int move_vertex_by(unsigned char vertex, Point *dp);
	//virtual iz object
	int move_vertex_to(unsigned char vertex, Point *dp);
	//virtual iz object
	Point *get_vertex_point(unsigned char vertex);
	//virtual iz object
	unsigned int get_vertex_distance(Point *tocka);

	Point *get_other_vertex_point(int vertex);

	//virtual iz primitive
	unsigned int get_distance(Point *tocka);
	//virtual iz primitive
	unsigned char on_object(Point *tocka);
	//virtual iz primitive
	void draw(FXDCWindow *hdc);

	//virtual iz object
	char *get_text();
	//virtual iz object
	void set_text(char *);

	//vrne kot
	float get_angle();
	//vrne dolzino
	double get_length();
	//vrne sredinsko tocko crte
	Point *get_center_of_line();
};
#endif //_OBJECT_H_