/*
This software uses the FOX Toolkit Library (http://www.fox-toolkit.org)

Copyright (c) 2004, Djukic Marko, Pernus Klemen, Zagar Miha

All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

   * Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in
     the documentation and/or other materials provided with the
     distribution.
   * Neither the name of the Faculty of Computer and Information Science 
     nor the names of its contributors may be used to endorse or promote
	 products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR 
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND 
FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/
#include "fx.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "icons.h" //ikonce, ki se pojavljajo v programu
#include "tuner.h" //okence kjer natuniras zadevo ...
#include "asaai.h" //the core .... :)

Asaai *asaai; //glaven modul prgrama, ki skrbi za vse skupaj

//glavno okecne
class AsaaiWindow : public FXMainWindow {
	FXDECLARE(AsaaiWindow)
private:
	FXVerticalFrame   *main_frame;
	FXHorizontalFrame *contents;				// glavni frame
	FXStatusbar *statusbar;						//statusna vrstica
	FXHorizontalFrame *tool_bar;				//vrstica z ikoncami
	FXVerticalFrame   *canvasFrame;             // Canvas frame
	FXVerticalFrame   *controlFrame;             // frame kjer so kontrole

	FXButton *show_constraints_button;	
	FXButton *hide_constraints_button;

	FXButton *color_button;	//trenutno izbrana barva
	FXButton *width_1_button; //izbira debeline crte
	FXButton *width_2_button; //izbira debeline crte
	FXButton *width_3_button; //izbira debeline crte
	FXButton *width_4_button; //izbira debeline crte
	FXButton *width_5_button; //izbira debeline crte
protected:
	AsaaiWindow(){} 
public:
	FXCanvas *canvas;                  // Canvas to draw into
	FXButton *text_button;
	FXFont *font;

	long on_paint(FXObject*,FXSelector,void*);
	
	long on_lmouse_down(FXObject*,FXSelector,void*);
	long on_lmouse_up(FXObject*,FXSelector,void*);
	long on_rmouse_down(FXObject*,FXSelector,void*);
	long on_rmouse_up(FXObject*,FXSelector,void*);
	long on_mouse_move(FXObject*,FXSelector,void*);

	long on_clear_canvas_button_click(FXObject*,FXSelector,void*);
	long on_show_constraints_button_click(FXObject*,FXSelector,void*);
	long on_hide_constraints_button_click(FXObject*,FXSelector,void*);

	long on_enter_text_button_click(FXObject*,FXSelector,void*);

	long on_tuner_button_click(FXObject*,FXSelector,void*);

	long on_color_button_click(FXObject*,FXSelector,void*);
	long on_width_button_click(FXObject*,FXSelector,void*);

	enum {
		ID_CANVAS=FXMainWindow::ID_LAST,
		ID_TUNER,
		ID_CLEAR_CANVAS,
		ID_SHOW_CONSTRAINTS,
		ID_HIDE_CONSTRAINTS,
		ID_COLORBUTTON,
		ID_WIDTH,
		ID_TEXT,
		ID_QUIT,
		ID_LAST
    };

	AsaaiWindow(FXApp* a);
	virtual void create();
};

FXDEFMAP(AsaaiWindow) AsaaiWindowMap[]={
  FXMAPFUNC(SEL_PAINT,				AsaaiWindow::ID_CANVAS,		AsaaiWindow::on_paint),
  FXMAPFUNC(SEL_LEFTBUTTONPRESS,	AsaaiWindow::ID_CANVAS,		AsaaiWindow::on_lmouse_down),
  FXMAPFUNC(SEL_LEFTBUTTONRELEASE,	AsaaiWindow::ID_CANVAS,		AsaaiWindow::on_lmouse_up),
  FXMAPFUNC(SEL_RIGHTBUTTONPRESS,   AsaaiWindow::ID_CANVAS,		AsaaiWindow::on_rmouse_down),
  FXMAPFUNC(SEL_RIGHTBUTTONRELEASE, AsaaiWindow::ID_CANVAS,		AsaaiWindow::on_rmouse_up),
  FXMAPFUNC(SEL_MOTION,				AsaaiWindow::ID_CANVAS,		AsaaiWindow::on_mouse_move),
  FXMAPFUNC(SEL_COMMAND,			AsaaiWindow::ID_CLEAR_CANVAS, AsaaiWindow::on_clear_canvas_button_click),
  FXMAPFUNC(SEL_COMMAND,			AsaaiWindow::ID_SHOW_CONSTRAINTS, AsaaiWindow::on_show_constraints_button_click),
  FXMAPFUNC(SEL_COMMAND,			AsaaiWindow::ID_HIDE_CONSTRAINTS, AsaaiWindow::on_hide_constraints_button_click),
  FXMAPFUNC(SEL_COMMAND,			AsaaiWindow::ID_TEXT, AsaaiWindow::on_enter_text_button_click),
  FXMAPFUNC(SEL_COMMAND,			AsaaiWindow::ID_TUNER, AsaaiWindow::on_tuner_button_click),
  FXMAPFUNC(SEL_COMMAND,			AsaaiWindow::ID_COLORBUTTON, AsaaiWindow::on_color_button_click),
  FXMAPFUNC(SEL_COMMAND,			AsaaiWindow::ID_WIDTH,		AsaaiWindow::on_width_button_click)
}; 

FXIMPLEMENT(AsaaiWindow,FXMainWindow,AsaaiWindowMap,ARRAYNUMBER(AsaaiWindowMap))


//zgradim okno ... canvas, gumbi, ...
AsaaiWindow::AsaaiWindow(FXApp *a): FXMainWindow(a,"Asaai - Artificial sense for aesthetic assisting illustration",NULL,NULL,DECOR_ALL,0,0,800,600){
	FXMatrix *matrix;
	FXGroupBox *gb;

	//statusna vrstica ...
	statusbar = new FXStatusbar(this,LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X);

	//glavno okence
	main_frame = new FXVerticalFrame(this,LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 0,0,0,0);

	//oroda vrstica :)
	tool_bar = new FXHorizontalFrame(main_frame,LAYOUT_SIDE_TOP|LAYOUT_FILL_X,0,0,0,0, 5,5,5,5);
	
	//clear button
	new FXButton(tool_bar,"\tClear screen\tClear screen.",new FXGIFIcon(getApp(),delete_gif,0,IMAGE_ALPHAGUESS),this,ID_CLEAR_CANVAS,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0,2,2,2,2);

	//separator
	new FXVerticalSeparator(tool_bar,LAYOUT_SIDE_RIGHT|LAYOUT_FILL_Y|SEPARATOR_GROOVE,0,0,0,0,10,10,0,0);

	//show/hide constrint button
	show_constraints_button = new FXButton(tool_bar,"\tShow constraints\tShow constraints.",new FXGIFIcon(getApp(),const_show_gif,0,IMAGE_ALPHAGUESS),this,ID_SHOW_CONSTRAINTS,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0,2,2,2,2);
	hide_constraints_button = new FXButton(tool_bar,"\tHide constraints\tHide constraints.",new FXGIFIcon(getApp(),const_hide_gif,0,IMAGE_ALPHAGUESS),this,ID_HIDE_CONSTRAINTS,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0,2,2,2,2);
	show_constraints_button->disable(); //na zacetku je show disablan ... 

	//separator
	new FXVerticalSeparator(tool_bar,LAYOUT_SIDE_RIGHT|LAYOUT_FILL_Y|SEPARATOR_GROOVE,0,0,0,0,10,10,0,0);

	//tunner
	new FXButton(tool_bar,"\tTune parameters\tTune parameters",new FXGIFIcon(getApp(),pref_gif,0,IMAGE_ALPHAGUESS),this,ID_TUNER,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0,2,2,2,2);


	//separator
	new FXVerticalSeparator(tool_bar,LAYOUT_SIDE_RIGHT|LAYOUT_FILL_Y|SEPARATOR_GROOVE,0,0,0,0,10,10,0,0);

	//text
	text_button = new FXButton(tool_bar,"\tEnter line text\tEnter line text",new FXGIFIcon(getApp(),text_gif,0,IMAGE_ALPHAGUESS),this,ID_TEXT,BUTTON_TOOLBAR|FRAME_RAISED,0,0,0,0,2,2,2,2);
	text_button->disable();


	//separator
	new FXVerticalSeparator(tool_bar,LAYOUT_SIDE_RIGHT|LAYOUT_FILL_Y|SEPARATOR_GROOVE,0,0,0,0,10,10,0,0);

	//quit button
	new FXButton(tool_bar,"\tQuit\tQuits asaai.",new FXGIFIcon(getApp(),quit_gif,0,IMAGE_ALPHAGUESS),getApp(),FXApp::ID_QUIT,BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_RIGHT,0,0,0,0,2,2,2,2);

	contents = new FXHorizontalFrame(main_frame,LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 0,0,0,0);

	// LEFT pane to contain the canvas
	canvasFrame = new FXVerticalFrame(contents,FRAME_SUNKEN|LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_TOP|LAYOUT_LEFT,0,0,0,0,10,10,10,10);
	//kamor risem zadeve
	canvas = new FXCanvas(canvasFrame,this,ID_CANVAS,FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_TOP|LAYOUT_LEFT);

	// RIGHT pane for the buttons
	controlFrame = new FXVerticalFrame(contents,FRAME_SUNKEN|LAYOUT_FILL_Y|LAYOUT_TOP|LAYOUT_LEFT,0,0,500,0,10,10,10,10);

	//group box za barve crt
	gb = new FXGroupBox(controlFrame,"",FRAME_RIDGE|LAYOUT_FILL_X);
	//izrisem bumb ki mi kaze kaksna je trenutno izbrana barva ... dvoklik na gumb zaene izbiratelja barv :)
	color_button = new FXButton(gb,"\t\tSelected line color",NULL,NULL,0,LAYOUT_CENTER_X,0,0,0,0,15,15,15,15);
	color_button->setBackColor(FXRGB(0,0,0));
	//matrix kjer se nahajajo gumki ze izbranih barv
	matrix = new FXMatrix(gb,2,MATRIX_BY_COLUMNS|LAYOUT_CENTER_X);
	//bumbki dolocenih barv
	(new FXButton(matrix,"\t\tSet line color",NULL,this,ID_COLORBUTTON,0,0,0,0,0,8,8,8,8))->setBackColor(FXRGB(0,0,0));
	(new FXButton(matrix,"\t\tSet line color",NULL,this,ID_COLORBUTTON,0,0,0,0,0,8,8,8,8))->setBackColor(FXRGB(127,127,127));
	(new FXButton(matrix,"\t\tSet line color",NULL,this,ID_COLORBUTTON,0,0,0,0,0,8,8,8,8))->setBackColor(FXRGB(255,0,0));
	(new FXButton(matrix,"\t\tSet line color",NULL,this,ID_COLORBUTTON,0,0,0,0,0,8,8,8,8))->setBackColor(FXRGB(255,255,0));
	(new FXButton(matrix,"\t\tSet line color",NULL,this,ID_COLORBUTTON,0,0,0,0,0,8,8,8,8))->setBackColor(FXRGB(255,0,255));
	(new FXButton(matrix,"\t\tSet line color",NULL,this,ID_COLORBUTTON,0,0,0,0,0,8,8,8,8))->setBackColor(FXRGB(0,255,0));
	(new FXButton(matrix,"\t\tSet line color",NULL,this,ID_COLORBUTTON,0,0,0,0,0,8,8,8,8))->setBackColor(FXRGB(0,255,255));
	(new FXButton(matrix,"\t\tSet line color",NULL,this,ID_COLORBUTTON,0,0,0,0,0,8,8,8,8))->setBackColor(FXRGB(0,0,255));


	//group box za debeline crt
	gb = new FXGroupBox(controlFrame,"",FRAME_RIDGE|LAYOUT_FILL_X);
	//izriem gumbe, ki mi kazejo debelino crt
	width_1_button = new FXButton(gb,"\t\tSet line width",new FXIcon(getApp(),line_width_1,FXRGB(255,255,255),0,40,1),this,ID_WIDTH,0,0,0,0,0,0,0,4,4);
	width_1_button->setBackColor(FXRGB(127,137,147)); //deafoltno izbran
	width_2_button = new FXButton(gb,"\t\tSet line width",new FXIcon(getApp(),line_width_2,FXRGB(255,255,255),0,40,2),this,ID_WIDTH,0,0,0,0,0,0,0,4,2);
	width_3_button = new FXButton(gb,"\t\tSet line width",new FXIcon(getApp(),line_width_3,FXRGB(255,255,255),0,40,3),this,ID_WIDTH,0,0,0,0,0,0,0,3,3);
	width_4_button = new FXButton(gb,"\t\tSet line width",new FXIcon(getApp(),line_width_4,FXRGB(255,255,255),0,40,4),this,ID_WIDTH,0,0,0,0,0,0,0,3,2);
	width_5_button = new FXButton(gb,"\t\tSet line width",new FXIcon(getApp(),line_width_5,FXRGB(255,255,255),0,40,5),this,ID_WIDTH,0,0,0,0,0,0,0,2,2);

	//pingvincek :)
	//new FXButton(controlFrame,"",new FXGIFIcon(getApp(),penguin,0,IMAGE_ALPHAGUESS),NULL,0,LAYOUT_BOTTOM,0,0,0,0,0,0,0,0);
} 

//dejanjsko zgradim in prikazem
void AsaaiWindow::create(){
	// Create the windows
	FXMainWindow::create();
	// Make the main window appear
	show();
}

// The mouse has moved, draw a line
long AsaaiWindow::on_mouse_move(FXObject*, FXSelector, void* ptr){
	FXEvent *ev = (FXEvent*)ptr;
	asaai->on_mouse_move(new Point(ev->win_x,ev->win_y));
  
	return 1;
} 

// Mouse button was pressed somewhere
long AsaaiWindow::on_lmouse_down(FXObject*, FXSelector, void *ptr){
	FXEvent *ev = (FXEvent*)ptr;
	
	if (ev->click_count == 2)
		asaai->on_lbutton_double(new Point(ev->win_x,ev->win_y));
	else
		asaai->on_lbutton_down(new Point(ev->win_x,ev->win_y));
  
	return 1;
} 

// The mouse button was released again
long AsaaiWindow::on_lmouse_up(FXObject*,FXSelector,void* ptr){
	FXEvent *ev = (FXEvent*)ptr;
	asaai->on_lbutton_up(new Point(ev->win_x,ev->win_y));
  
	return 1;
}

// Mouse button was pressed somewhere
long AsaaiWindow::on_rmouse_down(FXObject*, FXSelector, void *ptr){
	FXEvent *ev = (FXEvent*)ptr;

	if (ev->click_count == 2)
		asaai->on_rbutton_double(new Point(ev->win_x,ev->win_y));
	else  
		asaai->on_rbutton_down(new Point(ev->win_x,ev->win_y));
  
	return 1;
} 

// Mouse button was pressed somewhere
long AsaaiWindow::on_rmouse_up(FXObject*, FXSelector, void *ptr){
	FXEvent *ev = (FXEvent*)ptr;
	asaai->on_rbutton_up(new Point(ev->win_x,ev->win_y));
  
	return 1;
} 

  
// Paint the canvas
long AsaaiWindow::on_paint(FXObject*, FXSelector, void* ptr){
	asaai->on_paint(); //narisem na novo 

	return 1;
}


//klik na clear canvas button ... pobrise tisto kar je narisano na ekranu
long AsaaiWindow::on_clear_canvas_button_click(FXObject *obj, FXSelector, void* ptr) {
	asaai->clear();
	asaai->on_paint();

	return 1;
}

//klik na show constarints button
long AsaaiWindow::on_show_constraints_button_click(FXObject *obj, FXSelector, void* ptr) {
	if (asaai->show_constraints == 0) { //omejitve so nevidne ... jih ovidim
		asaai->show_constraints = 1;
		hide_constraints_button->enable();
		show_constraints_button->disable();
	}
	asaai->on_paint();
	return 1;
}
//klik na hide constarints button
long AsaaiWindow::on_hide_constraints_button_click(FXObject *obj, FXSelector, void* ptr) {
	if (asaai->show_constraints == 1) { //omejitve so vidne ... jih onevidim
		asaai->show_constraints = 0;
		show_constraints_button->enable();
		hide_constraints_button->disable();
	}
	asaai->on_paint();
	return 1;
}

//klik na text button
long AsaaiWindow::on_enter_text_button_click(FXObject *obj, FXSelector, void* ptr) {
	FXDialogBox dialog(getApp(),"Enter line text",DECOR_TITLE|DECOR_BORDER);
	const FXchar *line_text;
	
	Line *selected_object = NULL;
	list <Object*>::iterator iter_obj;

	for (iter_obj = asaai->object_list.begin(); iter_obj != asaai->object_list.end(); iter_obj++)
	{
		if ((*iter_obj)->is_selected() == 1){ //sem nasel izbran objekt
			selected_object = (Line*)*iter_obj;
			break;
		}
	}

	line_text = selected_object->get_text();



	FXVerticalFrame* content=new FXVerticalFrame(&dialog,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,10,10,10,10,10,10);
	new FXLabel(content,"Enter line text:",NULL,LAYOUT_FILL_X|JUSTIFY_LEFT);
	FXTextField *text=new FXTextField(content,30,&dialog,FXDialogBox::ID_ACCEPT,TEXTFIELD_ENTER_ONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X);
	new FXHorizontalSeparator(content,SEPARATOR_GROOVE|LAYOUT_FILL_X);
	FXHorizontalFrame* buttons=new FXHorizontalFrame(content,LAYOUT_FILL_X|PACK_UNIFORM_WIDTH,0,0,0,0,0,0,0,0);
	new FXButton(buttons,"&OK",NULL,&dialog,FXDialogBox::ID_ACCEPT,BUTTON_INITIAL|BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT);
	new FXButton(buttons,"&Cancel",NULL,&dialog,FXDialogBox::ID_CANCEL,BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT,0,0,0,0,20,20);
	if (line_text != NULL)
		text->setText(line_text);
	dialog.create();
	if(dialog.execute()){
		selected_object->set_text((char*)text->getText().text());
    }

	return 1;
}



//klice se tuner ... 
long AsaaiWindow::on_tuner_button_click(FXObject *obj, FXSelector, void* ptr) {
	TunerWindow tuner(this, asaai->get_ai(), asaai->get_recognizer());
	tuner.setX(getX()+80);
	tuner.setY(getY()+80);
	tuner.execute(PLACEMENT_DEFAULT);
 
	return 1;
}


//klik na enega od linw width gumbov ... dobim lin width ... mal grdo ampak nej bo :)
long AsaaiWindow::on_width_button_click(FXObject *obj, FXSelector, void* ptr) {
	int width;

	//jih odoznacim
	width_1_button->setBackColor(getApp()->getBaseColor());
	width_2_button->setBackColor(getApp()->getBaseColor());
	width_3_button->setBackColor(getApp()->getBaseColor());
	width_4_button->setBackColor(getApp()->getBaseColor());
	width_5_button->setBackColor(getApp()->getBaseColor());

	if (obj == 	width_1_button) {
			width_1_button->setBackColor(FXRGB(127,137,147));
			width = 1;
	}
	if (obj == 	width_2_button) {
			width_2_button->setBackColor(FXRGB(127,137,147));
			width = 2;
	}
	if (obj == 	width_3_button) {
			width_3_button->setBackColor(FXRGB(127,137,147));
			width = 3;
	}
	if (obj == 	width_4_button) {
			width_4_button->setBackColor(FXRGB(127,137,147));
			width = 4;
	}
	if (obj == 	width_5_button) {
			width_5_button->setBackColor(FXRGB(127,137,147));
			width = 5;
	}

	asaai->mouse_state->width = width;
	asaai->on_paint();

	return 1; 
}

//klik na enega od barvnih gumbov ... si zapomnik kaj sem izbral
long AsaaiWindow::on_color_button_click(FXObject *obj, FXSelector, void* ptr) {
	FXColor color;

	color = ((FXButton*)obj)->getBackColor();
	color_button->setBackColor(color);
	asaai->mouse_state->color->set_color(color & 255, (color >> 8) & 255, (color >> 16) & 255);

	asaai->on_paint();

	return 1; 
}




// Here we begin
int main(int argc, char *argv[]) 
{
	AsaaiWindow *main_window; //glavno okno programa

	// Make application
	FXApp* application=new FXApp("Asaai","");
	// Start app
	application->init(argc,argv);
	// Scribble window
	main_window = new AsaaiWindow(application);
	//ustvarim asaai modul ... podam mu canvas na katerega bo risal
	asaai = new Asaai(main_window->canvas,main_window->text_button);

	// Create the application's windows
	application->create();
	// Run the application
	application->run();
 
	return 0;
} 


